@@ -1,20 +1,8 @@
+from docstructure import SITE_STRUCTURE, HREF_MAP, BASENAME_MAP
 from lxml.etree import (parse, fromstring, ElementTree,
                         Element, SubElement, XPath)
 import os, shutil, re, sys, copy, time
 
-SITE_STRUCTURE = [
-    ('lxml', ('main.txt', 'intro.txt', 'lxml2.txt', 'FAQ.txt',
-              'compatibility.txt', 'performance.txt')),
-    ('Developing with lxml', ('tutorial.txt', '@API reference',
-                              'api.txt', 'parsing.txt',
-                              'validation.txt', 'xpathxslt.txt',
-                              'objectify.txt', 'lxmlhtml.txt',
-                              'cssselect.txt', 'elementsoup.txt')),
-    ('Extending lxml', ('resolvers.txt', 'extensions.txt',
-                        'element_classes.txt', 'sax.txt', 'capi.txt')),
-    ('Developing lxml', ('build.txt', 'lxml-source-howto.txt')),
-    ]
-
 RST2HTML_OPTIONS = " ".join([
     "--no-toc-backlinks",
     "--strip-comments",
@@ -22,23 +10,21 @@ RST2HTML_OPTIONS = " ".join([
     "--date",
     ])
 
-HREF_MAP = {
-    "API reference" : "api/index.html"
-}
-
 htmlnsmap = {"h" : "http://www.w3.org/1999/xhtml"}
 
 find_title = XPath("/h:html/h:head/h:title/text()", namespaces=htmlnsmap)
-find_headings = XPath("//h:h1[not(@class)]/h:a/text()", namespaces=htmlnsmap)
+find_headings = XPath("//h:h1[not(@class)]//text()", namespaces=htmlnsmap)
 find_menu = XPath("//h:ul[@id=$name]", namespaces=htmlnsmap)
 find_page_end = XPath("/h:html/h:body/h:div[last()]", namespaces=htmlnsmap)
 
+find_words = re.compile('(\w+)').findall
 replace_invalid = re.compile(r'[-_/.\s\\]').sub
 
 def make_menu_section_head(section, menuroot):
-    section_head = menuroot.xpath("//ul[@id=$section]/li", section=section)
+    section_id = section + '-section'
+    section_head = menuroot.xpath("//ul[@id=$section]/li", section=section_id)
     if not section_head:
-        ul = SubElement(menuroot, "ul", id=section)
+        ul = SubElement(menuroot, "ul", id=section_id)
         section_head = SubElement(ul, "li")
         title = SubElement(section_head, "span", {"class":"section title"})
         title.text = section
@@ -51,12 +37,12 @@ def build_menu(tree, basename, section_head):
     if page_title:
         page_title = page_title[0]
     else:
-        page_title = replace_invalid(' ', basename.capitalize())
+        page_title = replace_invalid('', basename.capitalize())
     build_menu_entry(page_title, basename+".html", section_head,
                      headings=find_headings(tree))
 
 def build_menu_entry(page_title, url, section_head, headings=None):
-    page_id = replace_invalid(' ', os.path.splitext(url)[0])
+    page_id = replace_invalid(' ', os.path.splitext(url)[0]) + '-menu'
     ul = SubElement(section_head, "ul", {"class":"menu foreign", "id":page_id})
 
     title = SubElement(ul, "li", {"class":"menu title"})
@@ -67,7 +53,7 @@ def build_menu_entry(page_title, url, section_head, headings=None):
         subul = SubElement(title, "ul", {"class":"submenu"})
         for heading in headings:
             li = SubElement(subul, "li", {"class":"menu item"})
-            ref = replace_invalid('-', heading.lower())
+            ref = '-'.join(find_words(replace_invalid(' ', heading.lower())))
             a  = SubElement(li, "a", href=url+'#'+ref)
             a.text = heading
 
@@ -78,7 +64,11 @@ def merge_menu(tree, menu, name):
         tag = el.tag
         if tag[0] != '{':
             el.tag = "{http://www.w3.org/1999/xhtml}" + tag
-    current_menu = find_menu(menu_root, name=name)
+    current_menu = find_menu(
+        menu_root, name=replace_invalid(' ', name + '-menu'))
+    if not current_menu:
+        current_menu = find_menu(
+            menu_root, name=replace_invalid('-', name + '-menu'))
     if current_menu:
         for submenu in current_menu:
             submenu.set("class", submenu.get("class", "").
@@ -102,6 +92,10 @@ def publish(dirname, lxml_path, release):
 
     shutil.copy(pubkey, dirname)
 
+    href_map = HREF_MAP.copy()
+    changelog_basename = 'changes-%s' % release
+    href_map['Release Changelog'] = changelog_basename + '.html'
+
     trees = {}
     menu = Element("div", {"class":"sidemenu"})
     # build HTML pages and parse them back
@@ -111,13 +105,12 @@ def publish(dirname, lxml_path, release):
             if filename.startswith('@'):
                 # special menu entry
                 page_title = filename[1:]
-                url = HREF_MAP[page_title]
+                url = href_map[page_title]
                 build_menu_entry(page_title, url, section_head)
             else:
                 path = os.path.join(doc_dir, filename)
-                basename = os.path.splitext(filename)[0]
-                if basename == 'main':
-                    basename = 'index'
+                basename = os.path.splitext(os.path.basename(filename))[0]
+                basename = BASENAME_MAP.get(basename, basename)
                 outname = basename + '.html'
                 outpath = os.path.join(dirname, outname)
 
@@ -128,20 +121,16 @@ def publish(dirname, lxml_path, release):
 
                 build_menu(tree, basename, section_head)
 
+    # also convert CHANGES.txt
+    rest2html(script,
+              os.path.join(lxml_path, 'CHANGES.txt'),
+              os.path.join(dirname, 'changes-%s.html' % release),
+              '')
+
     # integrate menu
     for tree, basename, outpath in trees.itervalues():
         new_tree = merge_menu(tree, menu, basename)
         new_tree.write(outpath)
 
-    # also convert INSTALL.txt and CHANGES.txt
-    rest2html(script,
-              os.path.join(lxml_path, 'INSTALL.txt'),
-              os.path.join(dirname, 'installation.html'),
-              stylesheet_url)
-    rest2html(script,
-              os.path.join(lxml_path, 'CHANGES.txt'),
-              os.path.join(dirname, 'changes-%s.html' % release),
-              stylesheet_url)
-
 if __name__ == '__main__':
     publish(sys.argv[1], sys.argv[2], sys.argv[3])
