@@ -5,17 +5,6 @@
 #define va_int(ap)     va_arg(ap, int)
 #define va_charptr(ap) va_arg(ap, char *)
 
-/* Py_ssize_t support was added in Python 2.5 */
-#if PY_VERSION_HEX < 0x02050000
-#  ifndef PY_SSIZE_T_MAX /* patched Pyrex? */
-     typedef int Py_ssize_t;
-#    define PY_SSIZE_T_MAX INT_MAX
-#    define PY_SSIZE_T_MIN INT_MIN
-#    define PyInt_FromSsize_t(z) PyInt_FromLong(z)
-#    define PyInt_AsSsize_t(o)   PyInt_AsLong(o)
-#  endif
-#endif
-
 /* Threading can crash under Python <= 2.4.1 */
 #if PY_VERSION_HEX < 0x02040200
 #  ifndef WITHOUT_THREADING
@@ -23,6 +12,26 @@
 #  endif
 #endif
 
+/* Python 3 doesn't have PyFile_*(), PyString_*(), ... */
+#if PY_VERSION_HEX >= 0x03000000
+#  define PyFile_AsFile(o)                   (NULL)
+#  define PyString_Check(o)                  PyBytes_Check(o)
+#  define PyString_CheckExact(o)             PyBytes_CheckExact(o)
+#  define PyString_FromStringAndSize(s, len) PyBytes_FromStringAndSize(s, len)
+#  define PyString_FromFormat                PyBytes_FromFormat
+#  define PyString_GET_SIZE(s)               PyBytes_GET_SIZE(s)
+#  define PyString_AS_STRING(s)              PyBytes_AS_STRING(s)
+#else
+/* we currently only use three parameters - MSVC can't compile (s, ...) */
+#  define PyUnicode_FromFormat(s, a, b) (NULL)
+#endif
+
+#if PY_VERSION_HEX >= 0x03000000
+#  define IS_PYTHON3 1
+#else
+#  define IS_PYTHON3 0
+#endif
+
 #ifdef WITHOUT_THREADING
 #  define PyEval_SaveThread() (NULL)
 #  define PyEval_RestoreThread(state)
@@ -70,10 +79,16 @@
 /* schematron was added in libxml2 2.6.21 */
 #ifdef LIBXML_SCHEMATRON_ENABLED
 #  define ENABLE_SCHEMATRON 1
+#  if LIBXML_VERSION < 20632
+     /* schematron error reporting was added in libxml2 2.6.32 */
+#    define xmlSchematronSetValidStructuredErrors(ctxt, errorfunc, data)
+#    define XML_SCHEMATRON_OUT_ERROR 0
+#  endif
 #else
 #  define ENABLE_SCHEMATRON 0
 #  define XML_SCHEMATRON_OUT_QUIET 0
 #  define XML_SCHEMATRON_OUT_XML 0
+#  define XML_SCHEMATRON_OUT_ERROR 0
    typedef void xmlSchematron;
    typedef void xmlSchematronParserCtxt;
    typedef void xmlSchematronValidCtxt;
@@ -85,19 +100,16 @@
 #  define xmlSchematronNewValidCtxt(schema, options) NULL
 #  define xmlSchematronValidateDoc(ctxt, doc) 0
 #  define xmlSchematronFreeValidCtxt(ctxt)
+#  define xmlSchematronSetValidStructuredErrors(ctxt, errorfunc, data)
 #endif
 
+
 /* work around MSDEV 6.0 */
 #if (_MSC_VER == 1200) && (WINVER < 0x0500)
 long _ftol( double ); //defined by VC6 C libs
 long _ftol2( double dblSource ) { return _ftol( dblSource ); }
 #endif
 
-/* Redefinition of some Python builtins as C functions */
-#define callable(o)     PyCallable_Check(o)
-#define _cstr(s)        PyString_AS_STRING(s)
-#define _fqtypename(o)  (((PyTypeObject*)o)->ob_type->tp_name)
-
 #ifdef __GNUC__
 /* Test for GCC > 2.95 */
 #if __GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)) 
@@ -109,17 +121,26 @@ long _ftol2( double dblSource ) { return _ftol( dblSource ); }
 #define unlikely_condition(x) (x)
 #endif /* __GNUC__ */
 
-static PyObject* __PY_NEW_GLOBAL_EMPTY_TUPLE = NULL;
+#ifndef Py_TYPE
+  #define Py_TYPE(ob)   (((PyObject*)(ob))->ob_type)
+#endif
 
 #define PY_NEW(T) \
      (((PyTypeObject*)(T))->tp_new( \
-         (PyTypeObject*)(T), \
-         (unlikely_condition(__PY_NEW_GLOBAL_EMPTY_TUPLE == NULL) ? \
-             (__PY_NEW_GLOBAL_EMPTY_TUPLE = PyTuple_New(0)) : \
-             (__PY_NEW_GLOBAL_EMPTY_TUPLE)), \
-         NULL))
+             (PyTypeObject*)(T), __pyx_empty_tuple, NULL))
 
-#define _isString(obj)   PyObject_TypeCheck(obj, &PyBaseString_Type)
+#define _fqtypename(o)  ((Py_TYPE(o))->tp_name)
+
+#if PY_MAJOR_VERSION < 3
+#define _isString(obj)   (PyString_CheckExact(obj)  || \
+                          PyUnicode_CheckExact(obj) || \
+                          PyObject_TypeCheck(obj, &PyBaseString_Type))
+#else
+#define _isString(obj)   (PyUnicode_CheckExact(obj) || \
+			  PyBytes_CheckExact(obj) || \
+			  PyUnicode_Check(obj) || \
+			  PyBytes_Check(obj))
+#endif
 
 #define _isElement(c_node) \
         (((c_node)->type == XML_ELEMENT_NODE) || \
