@@ -3,7 +3,11 @@
 
 import threading
 import re
-import urlparse
+try:
+    from urlparse import urljoin
+except ImportError:
+    # Python 3
+    from urllib.parse import urljoin
 import copy
 from lxml import etree
 from lxml.html import defs
@@ -17,7 +21,31 @@ except ImportError:
 try:
     set
 except NameError:
+    # Python 2.3
     from sets import Set as set
+try:
+    bytes = __builtins__["bytes"]
+except (KeyError, NameError):
+    # Python < 2.6
+    bytes = str
+try:
+    unicode = __builtins__["unicode"]
+except (KeyError, NameError):
+    # Python 3
+    unicode = str
+try:
+    basestring = __builtins__["basestring"]
+except (KeyError, NameError):
+    # Python 3
+    basestring = (str, bytes)
+
+def __fix_docstring(s):
+    import sys
+    if sys.version_info[0] >= 3:
+        sub = re.compile(r"^(\s*)u'", re.M).sub
+    else:
+        sub = re.compile(r"^(\s*)b'", re.M).sub
+    return sub(r"\1'", s)
 
 __all__ = [
     'document_fromstring', 'fragment_fromstring', 'fragments_fromstring', 'fromstring',
@@ -25,16 +53,40 @@ __all__ = [
     'find_rel_links', 'find_class', 'make_links_absolute',
     'resolve_base_href', 'iterlinks', 'rewrite_links', 'open_in_browser']
 
-_rel_links_xpath = etree.XPath("descendant-or-self::a[@rel]")
+XHTML_NAMESPACE = "http://www.w3.org/1999/xhtml"
+
+_rel_links_xpath = etree.XPath("descendant-or-self::a[@rel]|descendant-or-self::x:a[@rel]",
+                               namespaces={'x':XHTML_NAMESPACE})
+_options_xpath = etree.XPath("descendant-or-self::option|descendant-or-self::x:option",
+                             namespaces={'x':XHTML_NAMESPACE})
+_forms_xpath = etree.XPath("descendant-or-self::form|descendant-or-self::x:form",
+                           namespaces={'x':XHTML_NAMESPACE})
 #_class_xpath = etree.XPath(r"descendant-or-self::*[regexp:match(@class, concat('\b', $class_name, '\b'))]", {'regexp': 'http://exslt.org/regular-expressions'})
 _class_xpath = etree.XPath("descendant-or-self::*[@class and contains(concat(' ', normalize-space(@class), ' '), concat(' ', $class_name, ' '))]")
 _id_xpath = etree.XPath("descendant-or-self::*[@id=$id]")
 _collect_string_content = etree.XPath("string()")
 _css_url_re = re.compile(r'url\((.*?)\)', re.I)
 _css_import_re = re.compile(r'@import "(.*?)"')
-_label_xpath = etree.XPath("//label[@for=$id]")
+_label_xpath = etree.XPath("//label[@for=$id]|//x:label[@for=$id]",
+                           namespaces={'x':XHTML_NAMESPACE})
 _archive_re = re.compile(r'[^ ]+')
 
+def _transform_result(typ, result):
+    """Convert the result back into the input type.
+    """
+    if issubclass(typ, bytes):
+        return tostring(result, encoding='utf-8')
+    elif issubclass(typ, unicode):
+        return tostring(result, encoding=unicode)
+    else:
+        return result
+
+def _nons(tag):
+    if isinstance(tag, basestring):
+        if tag[0] == '{' and tag[1:len(XHTML_NAMESPACE)+1] == XHTML_NAMESPACE:
+            return tag.split('}')[-1]
+    return tag
+
 class HtmlMixin(object):
 
     def base_url(self):
@@ -51,7 +103,7 @@ class HtmlMixin(object):
         """
         Return a list of all the forms
         """
-        return list(self.getiterator('form'))
+        return _forms_xpath(self)
     forms = property(forms, doc=forms.__doc__)
 
     def body(self):
@@ -59,7 +111,7 @@ class HtmlMixin(object):
         Return the <body> element.  Can be called from a child element
         to get the document's head.
         """
-        return self.xpath('//body')[0]
+        return self.xpath('//body|//x:body', namespaces={'x':XHTML_NAMESPACE})[0]
     body = property(body, doc=body.__doc__)
 
     def head(self):
@@ -67,7 +119,7 @@ class HtmlMixin(object):
         Returns the <head> element.  Can be called from a child
         element to get the document's head.
         """
-        return self.xpath('//head')[0]
+        return self.xpath('//head|//x:head', namespaces={'x':XHTML_NAMESPACE})[0]
     head = property(head, doc=head.__doc__)
 
     def _label__get(self):
@@ -88,7 +140,7 @@ class HtmlMixin(object):
             raise TypeError(
                 "You cannot set a label for an element (%r) that has no id"
                 % self)
-        if not label.tag == 'label':
+        if _nons(label.tag) != 'label':
             raise TypeError(
                 "You can only assign label to a label element (not %r)"
                 % label)
@@ -124,7 +176,7 @@ class HtmlMixin(object):
 
             >>> h = fragment_fromstring('<div>Hello <b>World!</b></div>')
             >>> h.find('.//b').drop_tag()
-            >>> print tostring(h)
+            >>> print(tostring(h, encoding=unicode))
             <div>Hello World!</div>
         """
         parent = self.getparent()
@@ -180,7 +232,7 @@ class HtmlMixin(object):
             if default:
                 return default[0]
             else:
-                raise KeyError, id
+                raise KeyError(id)
 
     def text_content(self):
         """
@@ -221,7 +273,7 @@ class HtmlMixin(object):
         if resolve_base_href:
             self.resolve_base_href()
         def link_repl(href):
-            return urlparse.urljoin(base_url, href)
+            return urljoin(base_url, href)
         self.rewrite_links(link_repl)
 
     def resolve_base_href(self):
@@ -231,7 +283,7 @@ class HtmlMixin(object):
         tag once it has been applied.
         """
         base_href = None
-        basetags = self.xpath('//base[@href]')
+        basetags = self.xpath('//base[@href]|//x:base[@href]', namespaces={'x':XHTML_NAMESPACE})
         for b in basetags:
             base_href = b.get('href')
             b.drop_tree()
@@ -250,13 +302,14 @@ class HtmlMixin(object):
         link you get is exactly the link in the document.
         """
         link_attrs = defs.link_attrs
-        for el in self.getiterator():
+        for el in self.iter():
             attribs = el.attrib
-            if el.tag != 'object':
+            tag = _nons(el.tag)
+            if tag != 'object':
                 for attrib in link_attrs:
                     if attrib in attribs:
                         yield (el, attrib, attribs[attrib], 0)
-            elif el.tag == 'object':
+            elif tag == 'object':
                 codebase = None
                 ## <object> tags have attributes that are relative to
                 ## codebase
@@ -267,15 +320,15 @@ class HtmlMixin(object):
                     if attrib in attribs:
                         value = el.get(attrib)
                         if codebase is not None:
-                            value = urlparse.urljoin(codebase, value)
+                            value = urljoin(codebase, value)
                         yield (el, attrib, value, 0)
                 if 'archive' in attribs:
                     for match in _archive_re.finditer(el.get('archive')):
                         value = match.group(0)
                         if codebase is not None:
-                            value = urlparse.urljoin(codebase, value)
+                            value = urljoin(codebase, value)
                         yield (el, 'archive', value, match.start())
-            if el.tag == 'param':
+            if tag == 'param':
                 valuetype = el.get('valuetype') or ''
                 if valuetype.lower() == 'ref':
                     ## FIXME: while it's fine we *find* this link,
@@ -285,7 +338,7 @@ class HtmlMixin(object):
                     ## doesn't have a valuetype="ref" (which seems to be the norm)
                     ## http://www.w3.org/TR/html401/struct/objects.html#adef-valuetype
                     yield (el, 'value', el.get('value'), 0)
-            if el.tag == 'style' and el.text:
+            if tag == 'style' and el.text:
                 for match in _css_url_re.finditer(el.text):
                     yield (el, None, match.group(1), match.start(1))
                 for match in _css_import_re.finditer(el.text):
@@ -355,18 +408,17 @@ class _MethodFunc(object):
         self.copy = copy
         self.__doc__ = getattr(source_class, self.name).__doc__
     def __call__(self, doc, *args, **kw):
+        result_type = type(doc)
         if isinstance(doc, basestring):
             if 'copy' in kw:
                 raise TypeError(
                     "The keyword 'copy' can only be used with element inputs to %s, not a string input" % self.name)
-            return_string = True
             doc = fromstring(doc, **kw)
         else:
             if 'copy' in kw:
                 copy = kw.pop('copy')
             else:
                 copy = self.copy
-            return_string = False
             if copy:
                 doc = copy.deepcopy(doc)
         meth = getattr(doc, self.name)
@@ -374,10 +426,7 @@ class _MethodFunc(object):
         # FIXME: this None test is a bit sloppy 
         if result is None:
             # Then return what we got in
-            if return_string:
-                return tostring(doc)
-            else:
-                return doc
+            return _transform_result(result_type, doc)
         else:
             return result
 
@@ -446,14 +495,17 @@ class HtmlElementClassLookup(etree.CustomElementClassLookup):
 # parsing
 ################################################################################
 
-def document_fromstring(html, **kw):
-    value = etree.HTML(html, html_parser, **kw)
+def document_fromstring(html, parser=None, **kw):
+    if parser is None:
+        parser = html_parser
+    value = etree.fromstring(html, parser, **kw)
     if value is None:
         raise etree.ParserError(
             "Document is empty")
     return value
 
-def fragments_fromstring(html, no_leading_text=False, base_url=None, **kw):
+def fragments_fromstring(html, no_leading_text=False, base_url=None,
+                         parser=None, **kw):
     """
     Parses several HTML elements, returning a list of elements.
 
@@ -464,13 +516,15 @@ def fragments_fromstring(html, no_leading_text=False, base_url=None, **kw):
 
     base_url will set the document's base_url attribute (and the tree's docinfo.URL)
     """
+    if parser is None:
+        parser = html_parser
     # FIXME: check what happens when you give html with a body, head, etc.
     start = html[:20].lstrip().lower()
     if not start.startswith('<html') and not start.startswith('<!doctype'):
         html = '<html><body>%s</body></html>' % html
-    doc = document_fromstring(html, base_url=base_url, **kw)
-    assert doc.tag == 'html'
-    bodies = [e for e in doc if e.tag == 'body']
+    doc = document_fromstring(html, parser=parser, base_url=base_url, **kw)
+    assert _nons(doc.tag) == 'html'
+    bodies = [e for e in doc if _nons(e.tag) == 'body']
     assert len(bodies) == 1, ("too many bodies: %r in %r" % (bodies, html))
     body = bodies[0]
     elements = []
@@ -484,7 +538,8 @@ def fragments_fromstring(html, no_leading_text=False, base_url=None, **kw):
     # would be nice
     return elements
 
-def fragment_fromstring(html, create_parent=False, base_url=None, **kw):
+def fragment_fromstring(html, create_parent=False, base_url=None,
+                        parser=None, **kw):
     """
     Parses a single HTML element; it is an error if there is more than
     one element, or if anything but whitespace precedes or follows the
@@ -495,12 +550,16 @@ def fragment_fromstring(html, create_parent=False, base_url=None, **kw):
 
     base_url will set the document's base_url attribute (and the tree's docinfo.URL)
     """
+    if parser is None:
+        parser = html_parser
     if create_parent:
         if not isinstance(create_parent, basestring):
             create_parent = 'div'
         return fragment_fromstring('<%s>%s</%s>' % (
-            create_parent, html, create_parent), base_url=base_url, **kw)
-    elements = fragments_fromstring(html, no_leading_text=True, base_url=base_url, **kw)
+            create_parent, html, create_parent),
+                                   parser=parser, base_url=base_url, **kw)
+    elements = fragments_fromstring(html, parser=parser, no_leading_text=True,
+                                    base_url=base_url, **kw)
     if not elements:
         raise etree.ParserError(
             "No elements found")
@@ -515,7 +574,7 @@ def fragment_fromstring(html, create_parent=False, base_url=None, **kw):
     el.tail = None
     return el
 
-def fromstring(html, base_url=None, **kw):
+def fromstring(html, base_url=None, parser=None, **kw):
     """
     Parse the html, returning a single element/document.
 
@@ -524,13 +583,17 @@ def fromstring(html, base_url=None, **kw):
 
     base_url will set the document's base_url attribute (and the tree's docinfo.URL)
     """
+    if parser is None:
+        parser = html_parser
     start = html[:10].lstrip().lower()
     if start.startswith('<html') or start.startswith('<!doctype'):
         # Looks like a full HTML document
-        return document_fromstring(html, base_url=base_url, **kw)
+        return document_fromstring(html, parser=parser, base_url=base_url, **kw)
     # otherwise, lets parse it out...
-    doc = document_fromstring(html, base_url=base_url, **kw)
+    doc = document_fromstring(html, parser=parser, base_url=base_url, **kw)
     bodies = doc.findall('body')
+    if not bodies:
+        bodies = doc.findall('{%s}body' % XHTML_NAMESPACE)
     if bodies:
         body = bodies[0]
         if len(bodies) > 1:
@@ -549,6 +612,8 @@ def fromstring(html, base_url=None, **kw):
     else:
         body = None
     heads = doc.findall('head')
+    if not heads:
+        heads = doc.findall('{%s}head' % XHTML_NAMESPACE)
     if heads:
         # Well, we have some sort of structure, so lets keep it all
         head = heads[0]
@@ -588,8 +653,8 @@ def parse(filename_or_url, parser=None, base_url=None, **kw):
 def _contains_block_level_tag(el):
     # FIXME: I could do this with XPath, but would that just be
     # unnecessarily slow?
-    for el in el.getiterator():
-        if el.tag in defs.block_tags:
+    for el in el.iter():
+        if _nons(el.tag) in defs.block_tags:
             return True
     return False
 
@@ -599,7 +664,7 @@ def _element_name(el):
     elif isinstance(el, basestring):
         return 'string'
     else:
-        return el.tag
+        return _nons(el.tag)
 
 ################################################################################
 # form handling
@@ -646,7 +711,10 @@ class FormElement(HtmlElement):
             return self.get('name')
         elif self.get('id'):
             return '#' + self.get('id')
-        return str(self.body.findall('form').index(self))
+        forms = self.body.findall('form')
+        if not forms:
+            forms = self.body.findall('{%s}form' % XHTML_NAMESPACE)
+        return str(forms.index(self))
 
     def form_values(self):
         """
@@ -658,9 +726,10 @@ class FormElement(HtmlElement):
             name = el.name
             if not name:
                 continue
-            if el.tag == 'textarea':
+            tag = _nons(el.tag)
+            if tag == 'textarea':
                 results.append((name, el.value))
-            elif el.tag == 'select':
+            elif tag == 'select':
                 value = el.value
                 if el.multiple:
                     for v in value:
@@ -668,7 +737,7 @@ class FormElement(HtmlElement):
                 elif value is not None:
                     results.append((name, el.value))
             else:
-                assert el.tag == 'input', (
+                assert tag == 'input', (
                     "Unexpected tag: %r" % el)
                 if el.checkable and not el.checked:
                     continue
@@ -686,7 +755,7 @@ class FormElement(HtmlElement):
         base_url = self.base_url
         action = self.get('action')
         if base_url and action is not None:
-            return urlparse.urljoin(base_url, action)
+            return urljoin(base_url, action)
         else:
             return action
     def _action__set(self, value):
@@ -792,8 +861,8 @@ class InputGetter(object):
     checkboxes and radio elements are returned individually.
     """
 
-    _name_xpath = etree.XPath(".//*[@name = $name and (name(.) = 'select' or name(.) = 'input' or name(.) = 'textarea')]")
-    _all_xpath = etree.XPath(".//*[name() = 'select' or name() = 'input' or name() = 'textarea']")
+    _name_xpath = etree.XPath(".//*[@name = $name and (local-name(.) = 'select' or local-name(.) = 'input' or local-name(.) = 'textarea')]")
+    _all_xpath = etree.XPath(".//*[local-name() = 'select' or local-name() = 'input' or local-name() = 'textarea']")
 
     def __init__(self, form):
         self.form = form
@@ -832,8 +901,9 @@ class InputGetter(object):
     def keys(self):
         names = set()
         for el in self:
-            if el.name is not None:
-                names.add(el.name)
+            names.add(el.name)
+        if None in names:
+            names.remove(None)
         return list(names)
 
     def __iter__(self):
@@ -910,7 +980,7 @@ class SelectElement(InputMixin, HtmlElement):
         """
         if self.multiple:
             return MultipleSelectOptions(self)
-        for el in self.getiterator('option'):
+        for el in _options_xpath(self):
             if 'selected' in el.attrib:
                 value = el.get('value')
                 # FIXME: If value is None, what to return?, get_text()?
@@ -926,7 +996,7 @@ class SelectElement(InputMixin, HtmlElement):
             self.value.update(value)
             return
         if value is not None:
-            for el in self.getiterator('option'):
+            for el in _options_xpath(self):
                 # FIXME: also if el.get('value') is None?
                 if el.get('value') == value:
                     checked_option = el
@@ -934,7 +1004,7 @@ class SelectElement(InputMixin, HtmlElement):
             else:
                 raise ValueError(
                     "There is no option with the value of %r" % value)
-        for el in self.getiterator('option'):
+        for el in _options_xpath(self):
             if 'selected' in el.attrib:
                 del el.attrib['selected']
         if value is not None:
@@ -954,7 +1024,7 @@ class SelectElement(InputMixin, HtmlElement):
         All the possible values this select can have (the ``value``
         attribute of all the ``<option>`` elements.
         """
-        return [el.get('value') for el in self.getiterator('option')]
+        return [el.get('value') for el in _options_xpath(self)]
     value_options = property(value_options, doc=value_options.__doc__)
 
     def _multiple__get(self):
@@ -986,7 +1056,7 @@ class MultipleSelectOptions(SetMixin):
         """
         Iterator of all the ``<option>`` elements.
         """
-        return self.select.getiterator('option')
+        return iter(_options_xpath(self.select))
     options = property(options)
 
     def __iter__(self):
@@ -1269,6 +1339,34 @@ HtmlElementClassLookup._default_element_classes['label'] = LabelElement
 ## Serialization
 ############################################################
 
+def html_to_xhtml(html):
+    """Convert all tags in an HTML tree to XHTML by moving them to the
+    XHTML namespace.
+    """
+    try:
+        html = html.getroot()
+    except AttributeError:
+        pass
+    prefix = "{%s}" % XHTML_NAMESPACE
+    for el in html.iter():
+        tag = el.tag
+        if isinstance(tag, basestring):
+            if tag[0] != '{':
+                el.tag = prefix + tag
+
+def xhtml_to_html(xhtml):
+    """Convert all tags in an XHTML tree to HTML by removing their
+    XHTML namespace.
+    """
+    try:
+        xhtml = xhtml.getroot()
+    except AttributeError:
+        pass
+    prefix = "{%s}" % XHTML_NAMESPACE
+    prefix_len = len(prefix)
+    for el in xhtml.iter(prefix + "*"):
+        el.tag = el.tag[prefix_len:]
+
 # This isn't a general match, but it's a match for what libxml2
 # specifically serialises:
 __replace_meta_content_type = re.compile(
@@ -1299,15 +1397,15 @@ def tostring(doc, pretty_print=False, include_meta_content_type=False,
         >>> root = html.fragment_fromstring('<p>Hello<br>world!</p>')
 
         >>> html.tostring(root)
-        '<p>Hello<br>world!</p>'
+        b'<p>Hello<br>world!</p>'
         >>> html.tostring(root, method='html')
-        '<p>Hello<br>world!</p>'
+        b'<p>Hello<br>world!</p>'
 
         >>> html.tostring(root, method='xml')
-        '<p>Hello<br/>world!</p>'
+        b'<p>Hello<br/>world!</p>'
 
         >>> html.tostring(root, method='text')
-        'Helloworld!'
+        b'Helloworld!'
 
         >>> html.tostring(root, method='text', encoding=unicode)
         u'Helloworld!'
@@ -1318,6 +1416,8 @@ def tostring(doc, pretty_print=False, include_meta_content_type=False,
         html = __replace_meta_content_type('', html)
     return html
 
+tostring.__doc__ = __fix_docstring(tostring.__doc__)
+
 def open_in_browser(doc):
     """
     Open the HTML document in a web browser (saving it to a temporary
@@ -1332,7 +1432,7 @@ def open_in_browser(doc):
     fn = os.tempnam() + '.html'
     write_doc(fn, method="html")
     url = 'file://' + fn.replace(os.path.sep, '/')
-    print url
+    print(url)
     webbrowser.open(url)
     
 ################################################################################
@@ -1342,10 +1442,20 @@ def open_in_browser(doc):
 class HTMLParser(etree.HTMLParser):
     def __init__(self, **kwargs):
         super(HTMLParser, self).__init__(**kwargs)
-        self.setElementClassLookup(HtmlElementClassLookup())
+        self.set_element_class_lookup(HtmlElementClassLookup())
+
+class XHTMLParser(etree.XMLParser):
+    def __init__(self, **kwargs):
+        super(XHTMLParser, self).__init__(**kwargs)
+        self.set_element_class_lookup(HtmlElementClassLookup())
 
 def Element(*args, **kw):
+    """Create a new HTML Element.
+
+    This can also be used for XHTML documents.
+    """
     v = html_parser.makeelement(*args, **kw)
     return v
 
 html_parser = HTMLParser()
+xhtml_parser = XHTMLParser()
