@@ -2367,7 +2367,7 @@ class XIncludeTestCase(HelperTestCase):
 
         self.include(tree)
 
-        called = res_instance.called.items()
+        called = list(res_instance.called.items())
         called.sort()
         self.assertEquals(
             [("dtd", True), ("include", True), ("input", True)],
@@ -2403,6 +2403,8 @@ def test_suite():
         [make_doctest('../../../doc/tutorial.txt')])
     suite.addTests(
         [make_doctest('../../../doc/api.txt')])
+    suite.addTests(
+        [make_doctest('../../../doc/FAQ.txt')])
     suite.addTests(
         [make_doctest('../../../doc/parsing.txt')])
     suite.addTests(
