@@ -5,29 +5,38 @@ HTML parser test cases for etree
 """
 
 import unittest
-import tempfile, os
+import tempfile, os, os.path, sys
 
-from common_imports import StringIO, etree, fileInTestDir
+this_dir = os.path.dirname(__file__)
+if this_dir not in sys.path:
+    sys.path.insert(0, this_dir) # needed for Py3
+
+from common_imports import etree, StringIO, BytesIO, fileInTestDir, _bytes, _str
 from common_imports import SillyFileLike, HelperTestCase
 
+try:
+    unicode = __builtins__["unicode"]
+except (NameError, KeyError):
+    unicode = str
+
 class HtmlParserTestCase(HelperTestCase):
     """HTML parser test cases
     """
     etree = etree
 
-    html_str = "<html><head><title>test</title></head><body><h1>page title</h1></body></html>"
-    html_str_pretty = """\
+    html_str = _bytes("<html><head><title>test</title></head><body><h1>page title</h1></body></html>")
+    html_str_pretty = _bytes("""\
 <html>
 <head><title>test</title></head>
 <body><h1>page title</h1></body>
 </html>
-"""
-    broken_html_str = "<html><head><title>test<body><h1>page title</h3></p></html>"
-    uhtml_str = u"<html><head><title>test Ã¡\uF8D2</title></head><body><h1>page Ã¡\uF8D2 title</h1></body></html>"
+""")
+    broken_html_str = _bytes("<html><head><title>test<body><h1>page title</h3></p></html>")
+    uhtml_str = _str("<html><head><title>test Ã¡\uF8D2</title></head><body><h1>page Ã¡\uF8D2 title</h1></body></html>")
 
     def tearDown(self):
         super(HtmlParserTestCase, self).tearDown()
-        self.etree.setDefaultParser()
+        self.etree.set_default_parser()
 
     def test_module_HTML(self):
         element = self.etree.HTML(self.html_str)
@@ -47,7 +56,7 @@ class HtmlParserTestCase(HelperTestCase):
     def test_module_parse_html_error(self):
         parser = self.etree.HTMLParser(recover=False)
         parse = self.etree.parse
-        f = StringIO("<html></body>")
+        f = BytesIO("<html></body>")
         self.assertRaises(self.etree.XMLSyntaxError,
                           parse, f, parser)
 
@@ -148,37 +157,37 @@ class HtmlParserTestCase(HelperTestCase):
     def test_module_parse_html_norecover(self):
         parser = self.etree.HTMLParser(recover=False)
         parse = self.etree.parse
-        f = StringIO(self.broken_html_str)
+        f = BytesIO(self.broken_html_str)
         self.assertRaises(self.etree.XMLSyntaxError,
                           parse, f, parser)
 
     def test_parse_encoding_8bit_explicit(self):
-        text = u'Søk på nettet'
-        html_latin1 = (u'<p>%s</p>' % text).encode('iso-8859-1')
+        text = _str('Søk på nettet')
+        html_latin1 = (_str('<p>%s</p>') % text).encode('iso-8859-1')
 
         tree = self.etree.parse(
-            StringIO(html_latin1),
+            BytesIO(html_latin1),
             self.etree.HTMLParser(encoding="iso-8859-1"))
         p = tree.find("//p")
         self.assertEquals(p.text, text)
 
     def test_parse_encoding_8bit_override(self):
-        text = u'Søk på nettet'
-        wrong_head = '''
+        text = _str('Søk på nettet')
+        wrong_head = _str('''
         <head>
           <meta http-equiv="Content-Type"
                 content="text/html; charset=UTF-8" />
-        </head>'''
-        html_latin1 = (u'<html>%s<body><p>%s</p></body></html>' % (wrong_head,
-                                                                   text)
+        </head>''')
+        html_latin1 = (_str('<html>%s<body><p>%s</p></body></html>') % (wrong_head,
+                                                                        text)
                       ).encode('iso-8859-1')
 
         self.assertRaises(self.etree.ParseError,
                           self.etree.parse,
-                          StringIO(html_latin1))
+                          BytesIO(html_latin1))
 
         tree = self.etree.parse(
-            StringIO(html_latin1),
+            BytesIO(html_latin1),
             self.etree.HTMLParser(encoding="iso-8859-1"))
         p = tree.find("//p")
         self.assertEquals(p.text, text)
@@ -190,7 +199,7 @@ class HtmlParserTestCase(HelperTestCase):
 
     def test_module_HTML_cdata(self):
         # by default, libxml2 generates CDATA nodes for <script> content
-        html = '<html><head><style>foo</style></head></html>'
+        html = _bytes('<html><head><style>foo</style></head></html>')
         element = self.etree.HTML(html)
         self.assertEquals(element[0][0].text, "foo")
 
@@ -203,7 +212,7 @@ class HtmlParserTestCase(HelperTestCase):
         filename = tempfile.mktemp(suffix=".html")
         open(filename, 'wb').write(self.html_str)
         try:
-            f = open(filename, 'r')
+            f = open(filename, 'rb')
             tree = self.etree.parse(f, parser)
             f.close()
             self.assertEqual(self.etree.tostring(tree.getroot()), self.html_str)
@@ -233,22 +242,22 @@ class HtmlParserTestCase(HelperTestCase):
 
     def test_default_parser_HTML_broken(self):
         self.assertRaises(self.etree.XMLSyntaxError,
-                          self.etree.parse, StringIO(self.broken_html_str))
+                          self.etree.parse, BytesIO(self.broken_html_str))
 
-        self.etree.setDefaultParser( self.etree.HTMLParser() )
+        self.etree.set_default_parser( self.etree.HTMLParser() )
 
-        tree = self.etree.parse(StringIO(self.broken_html_str))
+        tree = self.etree.parse(BytesIO(self.broken_html_str))
         self.assertEqual(self.etree.tostring(tree.getroot()),
                          self.html_str)
 
-        self.etree.setDefaultParser()
+        self.etree.set_default_parser()
 
         self.assertRaises(self.etree.XMLSyntaxError,
-                          self.etree.parse, StringIO(self.broken_html_str))
+                          self.etree.parse, BytesIO(self.broken_html_str))
 
     def test_html_iterparse(self):
         iterparse = self.etree.iterparse
-        f = StringIO(
+        f = BytesIO(
             '<html><head><title>TITLE</title><body><p>P</p></body></html>')
 
         iterator = iterparse(f, html=True)
@@ -282,4 +291,4 @@ def test_suite():
     return suite
 
 if __name__ == '__main__':
-    print 'to test use test.py %s' % __file__
+    print('to test use test.py %s' % __file__)
