@@ -5,10 +5,15 @@ Tests specific to the lxml.objectify API
 """
 
 
-import unittest, operator, sys
+import unittest, operator, sys, os.path
 
-from common_imports import etree, StringIO, HelperTestCase, fileInTestDir
-from common_imports import SillyFileLike, canonicalize, doctest
+this_dir = os.path.dirname(__file__)
+if this_dir not in sys.path:
+    sys.path.insert(0, this_dir) # needed for Py3
+
+from common_imports import etree, HelperTestCase, fileInTestDir
+from common_imports import SillyFileLike, canonicalize, doctest, make_doctest
+from common_imports import _bytes, _str, StringIO, BytesIO
 
 from lxml import objectify
 
@@ -25,10 +30,9 @@ DEFAULT_NSMAP = { "py"  : PYTYPE_NAMESPACE,
 objectclass2xsitype = {
     # objectify built-in
     objectify.IntElement: ("int", "short", "byte", "unsignedShort",
-                           "unsignedByte",),
-    objectify.LongElement: ("integer", "nonPositiveInteger", "negativeInteger",
-                            "long", "nonNegativeInteger", "unsignedLong",
-                            "unsignedInt", "positiveInteger",),
+                           "unsignedByte", "integer", "nonPositiveInteger",
+                           "negativeInteger", "long", "nonNegativeInteger",
+                           "unsignedLong", "unsignedInt", "positiveInteger",),
     objectify.FloatElement: ("float", "double"),
     objectify.BoolElement: ("boolean",),
     objectify.StringElement: ("string", "normalizedString", "token", "language",
@@ -43,7 +47,6 @@ xsitype2objclass = dict([ (v, k) for k in objectclass2xsitype
 objectclass2pytype = {
     # objectify built-in
     objectify.IntElement: "int",
-    objectify.LongElement: "long",
     objectify.FloatElement: "float",
     objectify.BoolElement: "bool",
     objectify.StringElement: "str",
@@ -77,20 +80,30 @@ class ObjectifyTestCase(HelperTestCase):
         self.parser = self.etree.XMLParser(remove_blank_text=True)
         self.lookup = etree.ElementNamespaceClassLookup(
             objectify.ObjectifyElementClassLookup() )
-        self.parser.setElementClassLookup(self.lookup)
+        self.parser.set_element_class_lookup(self.lookup)
 
         self.Element = self.parser.makeelement
 
         ns = self.lookup.get_namespace("otherNS")
         ns[None] = self.etree.ElementBase
 
+        self._orig_types = objectify.getRegisteredTypes()
+
     def tearDown(self):
         self.lookup.get_namespace("otherNS").clear()
         objectify.set_pytype_attribute_tag()
         del self.lookup
         del self.parser
+
+        for pytype in objectify.getRegisteredTypes():
+            pytype.unregister()
+        for pytype in self._orig_types:
+            pytype.register()
+        del self._orig_types
+
         super(ObjectifyTestCase, self).tearDown()
 
+
     def test_element_nsmap_default(self):
         elt = objectify.Element("test")
         self.assertEquals(elt.nsmap, DEFAULT_NSMAP)
@@ -98,7 +111,7 @@ class ObjectifyTestCase(HelperTestCase):
     def test_element_nsmap_empty(self):
         nsmap = {}
         elt = objectify.Element("test", nsmap=nsmap)
-        self.assertEquals(elt.nsmap.values(), [PYTYPE_NAMESPACE])
+        self.assertEquals(list(elt.nsmap.values()), [PYTYPE_NAMESPACE])
 
     def test_element_nsmap_custom_prefixes(self):
         nsmap = {"mypy": PYTYPE_NAMESPACE,
@@ -154,7 +167,7 @@ class ObjectifyTestCase(HelperTestCase):
     def test_data_element_nsmap_empty(self):
         nsmap = {}
         value = objectify.DataElement("test this", nsmap=nsmap)
-        self.assertEquals(value.nsmap.values(), [PYTYPE_NAMESPACE])
+        self.assertEquals(list(value.nsmap.values()), [PYTYPE_NAMESPACE])
 
     def test_data_element_nsmap_custom_prefixes(self):
         nsmap = {"mypy": PYTYPE_NAMESPACE,
@@ -727,10 +740,20 @@ class ObjectifyTestCase(HelperTestCase):
         root = Element("{objectified}root")
         root.bool = True
         self.assertEquals(root.bool, True)
+        self.assertEquals(root.bool + root.bool, True + True)
+        self.assertEquals(True + root.bool, True + root.bool)
+        self.assertEquals(root.bool * root.bool, True * True)
+        self.assertEquals(int(root.bool), int(True))
+        self.assertEquals(complex(root.bool), complex(True))
         self.assert_(isinstance(root.bool, objectify.BoolElement))
 
         root.bool = False
         self.assertEquals(root.bool, False)
+        self.assertEquals(root.bool + root.bool, False + False)
+        self.assertEquals(False + root.bool, False + root.bool)
+        self.assertEquals(root.bool * root.bool, False * False)
+        self.assertEquals(int(root.bool), int(False))
+        self.assertEquals(complex(root.bool), complex(False))
         self.assert_(isinstance(root.bool, objectify.BoolElement))
 
     def test_data_element_bool(self):
@@ -805,6 +828,21 @@ class ObjectifyTestCase(HelperTestCase):
         el = objectify.DataElement(s)
         val = 5
         self.assertRaises(TypeError, el.__mod__, val)
+
+    def test_type_str_as_int(self):
+        v = "1"
+        el = objectify.DataElement(v)
+        self.assertEquals(int(el), 1)
+            
+    def test_type_str_as_float(self):
+        v = "1"
+        el = objectify.DataElement(v)
+        self.assertEquals(float(el), 1)
+
+    def test_type_str_as_complex(self):
+        v = "1"
+        el = objectify.DataElement(v)
+        self.assertEquals(complex(el), 1)
             
     def test_type_str_mod_data_elements(self):
         s = "%d %f %s %r"
@@ -834,59 +872,59 @@ class ObjectifyTestCase(HelperTestCase):
         Element = self.Element
         SubElement = self.etree.SubElement
         root = Element("{objectified}root")
-        root.s = u"test"
+        root.s = _str("test")
         self.assert_(isinstance(root.s, objectify.StringElement))
 
     def test_type_ustr_intliteral(self):
         Element = self.Element
         SubElement = self.etree.SubElement
         root = Element("{objectified}root")
-        root.s = u"3"
+        root.s = _str("3")
         self.assert_(isinstance(root.s, objectify.StringElement))
 
     def test_type_ustr_floatliteral(self):
         Element = self.Element
         SubElement = self.etree.SubElement
         root = Element("{objectified}root")
-        root.s = u"3.72"
+        root.s = _str("3.72")
         self.assert_(isinstance(root.s, objectify.StringElement))
 
     def test_type_ustr_mul(self):
         Element = self.Element
         SubElement = self.etree.SubElement
         root = Element("{objectified}root")
-        root.s = u"test"
+        root.s = _str("test")
 
-        self.assertEquals(u"test" * 5, root.s * 5)
-        self.assertEquals(5 * u"test", 5 * root.s)
+        self.assertEquals(_str("test") * 5, root.s * 5)
+        self.assertEquals(5 * _str("test"), 5 * root.s)
 
-        self.assertRaises(TypeError, operator.mul, root.s, u"honk")
-        self.assertRaises(TypeError, operator.mul, u"honk", root.s)
+        self.assertRaises(TypeError, operator.mul, root.s, _str("honk"))
+        self.assertRaises(TypeError, operator.mul, _str("honk"), root.s)
 
     def test_type_ustr_add(self):
         Element = self.Element
         SubElement = self.etree.SubElement
         root = Element("{objectified}root")
-        root.s = u"test"
+        root.s = _str("test")
 
-        s = u"toast"
-        self.assertEquals(u"test" + s, root.s + s)
-        self.assertEquals(s + u"test", s + root.s)
+        s = _str("toast")
+        self.assertEquals(_str("test") + s, root.s + s)
+        self.assertEquals(s + _str("test"), s + root.s)
 
     def test_data_element_ustr(self):
-        value = objectify.DataElement(u"test")
+        value = objectify.DataElement(_str("test"))
         self.assert_(isinstance(value, objectify.StringElement))
-        self.assertEquals(value, u"test")
+        self.assertEquals(value, _str("test"))
 
     def test_data_element_ustr_intliteral(self):
         value = objectify.DataElement("3")
         self.assert_(isinstance(value, objectify.StringElement))
-        self.assertEquals(value, u"3")
+        self.assertEquals(value, _str("3"))
 
     def test_data_element_ustr_floatliteral(self):
-        value = objectify.DataElement(u"3.20")
+        value = objectify.DataElement(_str("3.20"))
         self.assert_(isinstance(value, objectify.StringElement))
-        self.assertEquals(value, u"3.20")
+        self.assertEquals(value, _str("3.20"))
 
     def test_type_int(self):
         Element = self.Element
@@ -913,7 +951,7 @@ class ObjectifyTestCase(HelperTestCase):
         self.assertEquals(value, 5.5)
 
     def test_data_element_xsitypes(self):
-        for xsi, objclass in xsitype2objclass.iteritems():
+        for xsi, objclass in xsitype2objclass.items():
             # 1 is a valid value for all ObjectifiedDataElement classes
             pyval = 1
             value = objectify.DataElement(pyval, _xsi=xsi)
@@ -922,7 +960,7 @@ class ObjectifyTestCase(HelperTestCase):
                          % (pyval, xsi, type(value), objclass))
         
     def test_data_element_xsitypes_xsdprefixed(self):
-        for xsi, objclass in xsitype2objclass.iteritems():
+        for xsi, objclass in xsitype2objclass.items():
             # 1 is a valid value for all ObjectifiedDataElement classes
             pyval = 1
             value = objectify.DataElement(pyval, _xsi="xsd:%s" % xsi)
@@ -931,13 +969,13 @@ class ObjectifyTestCase(HelperTestCase):
                          % (pyval, xsi, type(value), objclass))
         
     def test_data_element_xsitypes_prefixed(self):
-        for xsi, objclass in xsitype2objclass.iteritems():
+        for xsi, objclass in xsitype2objclass.items():
             # 1 is a valid value for all ObjectifiedDataElement classes
             self.assertRaises(ValueError, objectify.DataElement, 1,
                               _xsi="foo:%s" % xsi)
 
     def test_data_element_pytypes(self):
-        for pytype, objclass in pytype2objclass.iteritems():
+        for pytype, objclass in pytype2objclass.items():
             # 1 is a valid value for all ObjectifiedDataElement classes
             pyval = 1
             value = objectify.DataElement(pyval, _pytype=pytype)
@@ -1043,13 +1081,13 @@ class ObjectifyTestCase(HelperTestCase):
             self.assert_(isinstance(s, objectify.StringElement))
             self.assertEquals("5", s)
 
-        for l in root.l:
-            self.assert_(isinstance(l, objectify.LongElement))
-            self.assertEquals(5L, l)
-
         for i in root.i:
             self.assert_(isinstance(i, objectify.IntElement))
             self.assertEquals(5, i)
+
+        for l in root.l:
+            self.assert_(isinstance(l, objectify.IntElement))
+            self.assertEquals(5, i)
             
         self.assert_(isinstance(root.n, objectify.NoneElement))
         self.assertEquals(None, root.n)
@@ -1112,27 +1150,27 @@ class ObjectifyTestCase(HelperTestCase):
             self.assert_(isinstance(s, objectify.StringElement))
             self.assertEquals("5", s)
 
-        for l in root.l:
-            self.assert_(isinstance(l, objectify.LongElement))
-            self.assertEquals(5L, l)
-
         for i in root.i:
             self.assert_(isinstance(i, objectify.IntElement))
             self.assertEquals(5, i)
+
+        for l in root.l:
+            self.assert_(isinstance(l, objectify.IntElement))
+            self.assertEquals(5, l)
             
         self.assert_(isinstance(root.n, objectify.NoneElement))
         self.assertEquals(None, root.n)
         
     def test_type_str_sequence(self):
         XML = self.XML
-        root = XML(u'<root><b>why</b><b>try</b></root>')
+        root = XML(_bytes('<root><b>why</b><b>try</b></root>'))
         strs = [ str(s) for s in root.b ]
         self.assertEquals(["why", "try"],
                           strs)
 
     def test_type_str_cmp(self):
         XML = self.XML
-        root = XML(u'<root><b>test</b><b>taste</b><b></b><b/></root>')
+        root = XML(_bytes('<root><b>test</b><b>taste</b><b></b><b/></root>'))
         self.assertFalse(root.b[0] <  root.b[1])
         self.assertFalse(root.b[0] <= root.b[1])
         self.assertFalse(root.b[0] == root.b[1])
@@ -1143,8 +1181,6 @@ class ObjectifyTestCase(HelperTestCase):
 
         self.assertEquals(root.b[0], "test")
         self.assertEquals("test", root.b[0])
-        self.assert_(root.b[0] >  5)
-        self.assert_(5 < root.b[0])
 
         self.assertEquals("", root.b[2])
         self.assertEquals(root.b[2], "")
@@ -1161,7 +1197,7 @@ class ObjectifyTestCase(HelperTestCase):
 
     def test_type_int_cmp(self):
         XML = self.XML
-        root = XML(u'<root><b>5</b><b>6</b></root>')
+        root = XML(_bytes('<root><b>5</b><b>6</b></root>'))
         self.assert_(root.b[0] <  root.b[1])
         self.assert_(root.b[0] <= root.b[1])
         self.assert_(root.b[0] != root.b[1])
@@ -1172,8 +1208,7 @@ class ObjectifyTestCase(HelperTestCase):
 
         self.assertEquals(root.b[0], 5)
         self.assertEquals(5, root.b[0])
-        self.assert_(root.b[0] <  "5")
-        self.assert_("5" > root.b[0])
+        self.assertNotEquals(root.b[0], "5")
 
         root.b = 5
         self.assert_(root.b)
@@ -1184,7 +1219,7 @@ class ObjectifyTestCase(HelperTestCase):
 
     def test_type_bool_cmp(self):
         XML = self.XML
-        root = XML(u'<root><b>false</b><b>true</b></root>')
+        root = XML(_bytes('<root><b>false</b><b>true</b></root>'))
         self.assert_(root.b[0] <  root.b[1])
         self.assert_(root.b[0] <= root.b[1])
         self.assert_(root.b[0] != root.b[1])
@@ -1208,21 +1243,23 @@ class ObjectifyTestCase(HelperTestCase):
 
     def test_type_none_cmp(self):
         XML = self.XML
-        root = XML(u"""
+        root = XML(_bytes("""
         <root xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
           <b xsi:nil="true"></b><b xsi:nil="true"/>
-        </root>""")
+        </root>"""))
         self.assert_(root.b[0] == root.b[1])
         self.assertFalse(root.b[0])
         self.assertEquals(root.b[0], None)
         self.assertEquals(None, root.b[0])
 
-        for comparison in ["abc", 5, 7.3, True, [], ()]:
-            none = root.b[1]
-            self.assert_(none < comparison, "%s (%s) should be < %s" %
-                         (none, type(none), comparison) )
-            self.assert_(comparison > none, "%s should be > %s (%s)" %
-                         (comparison, none, type(none)) )
+        # doesn't work in Py3:
+
+        #for comparison in ["abc", 5, 7.3, True, [], ()]:
+        #    none = root.b[1]
+        #    self.assert_(none < comparison, "%s (%s) should be < %s" %
+        #                 (none, type(none), comparison) )
+        #    self.assert_(comparison > none, "%s should be > %s (%s)" %
+        #                 (comparison, none, type(none)) )
 
     def test_dataelement_xsi(self):
         el = objectify.DataElement(1, _xsi="string")
@@ -1243,7 +1280,7 @@ class ObjectifyTestCase(HelperTestCase):
 
     def test_pytype_annotation(self):
         XML = self.XML
-        root = XML(u'''\
+        root = XML(_bytes('''\
         <a xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xmlns:py="http://codespeak.net/lxml/objectify/pytype">
           <b>5</b>
@@ -1261,7 +1298,7 @@ class ObjectifyTestCase(HelperTestCase):
           <l py:pytype="long">2</l>
           <t py:pytype="TREE"></t>
         </a>
-        ''')
+        '''))
         objectify.annotate(root)
 
         child_types = [ c.get(objectify.PYTYPE_ATTRIBUTE)
@@ -1285,12 +1322,12 @@ class ObjectifyTestCase(HelperTestCase):
 
     def test_pytype_annotation_empty(self):
         XML = self.XML
-        root = XML(u'''\
+        root = XML(_bytes('''\
         <a xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xmlns:py="http://codespeak.net/lxml/objectify/pytype">
           <n></n>
         </a>
-        ''')
+        '''))
         objectify.annotate(root)
 
         child_types = [ c.get(objectify.PYTYPE_ATTRIBUTE)
@@ -1305,7 +1342,7 @@ class ObjectifyTestCase(HelperTestCase):
 
     def test_pytype_annotation_use_old(self):
         XML = self.XML
-        root = XML(u'''\
+        root = XML(_bytes('''\
         <a xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xmlns:py="http://codespeak.net/lxml/objectify/pytype">
           <b>5</b>
@@ -1323,7 +1360,7 @@ class ObjectifyTestCase(HelperTestCase):
           <l py:pytype="long">2</l>
           <t py:pytype="TREE"></t>
         </a>
-        ''')
+        '''))
         objectify.annotate(root, ignore_old=False)
 
         child_types = [ c.get(objectify.PYTYPE_ATTRIBUTE)
@@ -1340,14 +1377,14 @@ class ObjectifyTestCase(HelperTestCase):
         self.assertEquals("str",   child_types[ 9])
         self.assertEquals("str",   child_types[10])
         self.assertEquals("float", child_types[11])
-        self.assertEquals("long",  child_types[12])
+        self.assertEquals("int",   child_types[12])
         self.assertEquals(TREE_PYTYPE,  child_types[13])
         
         self.assertEquals("true", root.n.get(XML_SCHEMA_NIL_ATTR))
 
     def test_pytype_xsitype_annotation(self):
         XML = self.XML
-        root = XML(u'''\
+        root = XML(_bytes('''\
         <a xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xmlns:py="http://codespeak.net/lxml/objectify/pytype">
           <b>5</b>
@@ -1365,7 +1402,7 @@ class ObjectifyTestCase(HelperTestCase):
           <l py:pytype="long">2</l>
           <t py:pytype="TREE"></t>
         </a>
-        ''')
+        '''))
         objectify.annotate(root, ignore_old=False, ignore_xsi=False,
                            annotate_xsi=1, annotate_pytype=1)
         
@@ -1384,7 +1421,7 @@ class ObjectifyTestCase(HelperTestCase):
         self.assertEquals("str",   child_types[ 9])
         self.assertEquals("str",   child_types[10])
         self.assertEquals("float",   child_types[11])
-        self.assertEquals("long",   child_types[12])
+        self.assertEquals("int",     child_types[12])
         self.assertEquals(TREE_PYTYPE,  child_types[13])
         
         self.assertEquals("true", root.n.get(XML_SCHEMA_NIL_ATTR))
@@ -1395,7 +1432,7 @@ class ObjectifyTestCase(HelperTestCase):
         # check xsi annotations
         child_types = [ c.get(XML_SCHEMA_INSTANCE_TYPE_ATTR)
                         for c in root.iterchildren() ]
-        self.assertEquals("xsd:int",     child_types[ 0])
+        self.assertEquals("xsd:integer", child_types[ 0])
         self.assertEquals("xsd:string",  child_types[ 1])
         self.assertEquals("xsd:double",  child_types[ 2])
         self.assertEquals("xsd:string",  child_types[ 3])
@@ -1414,7 +1451,7 @@ class ObjectifyTestCase(HelperTestCase):
 
     def test_xsiannotate_use_old(self):
         XML = self.XML
-        root = XML(u'''\
+        root = XML(_bytes('''\
         <a xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xmlns:py="http://codespeak.net/lxml/objectify/pytype">
           <b>5</b>
@@ -1432,12 +1469,12 @@ class ObjectifyTestCase(HelperTestCase):
           <l py:pytype="long">2</l>
           <t py:pytype="TREE"></t>
         </a>
-        ''')
+        '''))
         objectify.xsiannotate(root, ignore_old=False)
 
         child_types = [ c.get(XML_SCHEMA_INSTANCE_TYPE_ATTR)
                         for c in root.iterchildren() ]
-        self.assertEquals("xsd:int",     child_types[ 0])
+        self.assertEquals("xsd:integer", child_types[ 0])
         self.assertEquals("xsd:string",  child_types[ 1])
         self.assertEquals("xsd:double",  child_types[ 2])
         self.assertEquals("xsd:string",  child_types[ 3])
@@ -1454,7 +1491,7 @@ class ObjectifyTestCase(HelperTestCase):
 
     def test_pyannotate_ignore_old(self):
         XML = self.XML
-        root = XML(u'''\
+        root = XML(_bytes('''\
         <a xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xmlns:py="http://codespeak.net/lxml/objectify/pytype">
           <b>5</b>
@@ -1472,7 +1509,7 @@ class ObjectifyTestCase(HelperTestCase):
           <l py:pytype="long">2</l>
           <t py:pytype="TREE"></t>
         </a>
-        ''')
+        '''))
         objectify.pyannotate(root, ignore_old=True)
 
         child_types = [ c.get(objectify.PYTYPE_ATTRIBUTE)
@@ -1496,7 +1533,7 @@ class ObjectifyTestCase(HelperTestCase):
 
     def test_pyannotate_empty(self):
         XML = self.XML
-        root = XML(u'''\
+        root = XML('''\
         <a xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xmlns:py="http://codespeak.net/lxml/objectify/pytype">
           <n></n>
@@ -1516,7 +1553,7 @@ class ObjectifyTestCase(HelperTestCase):
 
     def test_pyannotate_use_old(self):
         XML = self.XML
-        root = XML(u'''\
+        root = XML('''\
         <a xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xmlns:py="http://codespeak.net/lxml/objectify/pytype">
           <b>5</b>
@@ -1551,14 +1588,14 @@ class ObjectifyTestCase(HelperTestCase):
         self.assertEquals("str",   child_types[ 9])
         self.assertEquals("str",   child_types[10])
         self.assertEquals("float", child_types[11])
-        self.assertEquals("long",  child_types[12])
+        self.assertEquals("int",   child_types[12])
         self.assertEquals(TREE_PYTYPE, child_types[13])
         
         self.assertEquals("true", root.n.get(XML_SCHEMA_NIL_ATTR))
         
     def test_xsiannotate_ignore_old(self):
         XML = self.XML
-        root = XML(u'''\
+        root = XML(_bytes('''\
         <a xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xmlns:py="http://codespeak.net/lxml/objectify/pytype">
           <b>5</b>
@@ -1576,21 +1613,21 @@ class ObjectifyTestCase(HelperTestCase):
           <l py:pytype="long">2</l>
           <t py:pytype="TREE"></t>
         </a>
-        ''')
+        '''))
         objectify.xsiannotate(root, ignore_old=True)
 
         child_types = [ c.get(XML_SCHEMA_INSTANCE_TYPE_ATTR)
                         for c in root.iterchildren() ]
-        self.assertEquals("xsd:int",     child_types[ 0])
+        self.assertEquals("xsd:integer", child_types[ 0])
         self.assertEquals("xsd:string",  child_types[ 1])
         self.assertEquals("xsd:double",  child_types[ 2])
         self.assertEquals("xsd:string",  child_types[ 3])
         self.assertEquals("xsd:boolean", child_types[ 4])
         self.assertEquals(None,          child_types[ 5])
         self.assertEquals(None,          child_types[ 6])
-        self.assertEquals("xsd:int",     child_types[ 7])
-        self.assertEquals("xsd:int",     child_types[ 8])
-        self.assertEquals("xsd:int",     child_types[ 9])
+        self.assertEquals("xsd:integer", child_types[ 7])
+        self.assertEquals("xsd:integer", child_types[ 8])
+        self.assertEquals("xsd:integer", child_types[ 9])
         self.assertEquals("xsd:string",  child_types[10])
         self.assertEquals("xsd:double",  child_types[11])
         self.assertEquals("xsd:integer", child_types[12])
@@ -1600,7 +1637,7 @@ class ObjectifyTestCase(HelperTestCase):
 
     def test_deannotate(self):
         XML = self.XML
-        root = XML(u'''\
+        root = XML(_bytes('''\
         <a xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xmlns:py="http://codespeak.net/lxml/objectify/pytype">
           <b>5</b>
@@ -1618,7 +1655,7 @@ class ObjectifyTestCase(HelperTestCase):
           <l py:pytype="long">2</l>
           <t py:pytype="TREE"></t>
         </a>
-        ''')
+        '''))
         objectify.deannotate(root)
 
         for c in root.getiterator():
@@ -1629,7 +1666,7 @@ class ObjectifyTestCase(HelperTestCase):
 
     def test_pytype_deannotate(self):
         XML = self.XML
-        root = XML(u'''\
+        root = XML(_bytes('''\
         <a xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xmlns:py="http://codespeak.net/lxml/objectify/pytype">
           <b>5</b>
@@ -1647,7 +1684,7 @@ class ObjectifyTestCase(HelperTestCase):
           <l py:pytype="long">2</l>
           <t py:pytype="TREE"></t>
         </a>
-        ''')
+        '''))
         objectify.xsiannotate(root)
         objectify.deannotate(root, xsi=False)
 
@@ -1675,7 +1712,7 @@ class ObjectifyTestCase(HelperTestCase):
 
     def test_xsitype_deannotate(self):
         XML = self.XML
-        root = XML(u'''\
+        root = XML(_bytes('''\
         <a xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xmlns:py="http://codespeak.net/lxml/objectify/pytype"
         xmlns:xsd="http://www.w3.org/2001/XMLSchema">
@@ -1694,7 +1731,7 @@ class ObjectifyTestCase(HelperTestCase):
           <l py:pytype="long">2</l>
           <t py:pytype="TREE"></t>
         </a>
-        ''')
+        '''))
         objectify.annotate(root)
         objectify.deannotate(root, pytype=False)
 
@@ -1722,7 +1759,7 @@ class ObjectifyTestCase(HelperTestCase):
 
     def test_pytype_deannotate(self):
         XML = self.XML
-        root = XML(u'''\
+        root = XML(_bytes('''\
         <a xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xmlns:py="http://codespeak.net/lxml/objectify/pytype"
         xmlns:xsd="http://www.w3.org/2001/XMLSchema">
@@ -1741,7 +1778,7 @@ class ObjectifyTestCase(HelperTestCase):
           <l xsi:type="xsd:long">2</l>
           <t py:pytype="TREE"></t>
         </a>
-        ''')
+        '''))
         objectify.annotate(root)
         objectify.deannotate(root, xsi=False)
 
@@ -1770,7 +1807,7 @@ class ObjectifyTestCase(HelperTestCase):
     def test_change_pytype_attribute(self):
         XML = self.XML
 
-        xml = u'''\
+        xml = _bytes('''\
         <a xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
           <b>5</b>
           <b>test</b>
@@ -1781,7 +1818,7 @@ class ObjectifyTestCase(HelperTestCase):
           <n></n>
           <b xsi:type="double">5</b>
         </a>
-        '''
+        ''')
 
         pytype_ns, pytype_name = objectify.PYTYPE_ATTRIBUTE[1:].split('}')
         objectify.set_pytype_attribute_tag("{TEST}test")
@@ -1814,39 +1851,80 @@ class ObjectifyTestCase(HelperTestCase):
 
     def test_registered_types(self):
         orig_types = objectify.getRegisteredTypes()
+        orig_types[0].unregister()
+        self.assertEquals(orig_types[1:], objectify.getRegisteredTypes())
 
-        try:
-            orig_types[0].unregister()
-            self.assertEquals(orig_types[1:], objectify.getRegisteredTypes())
+        class NewType(objectify.ObjectifiedDataElement):
+            pass
 
-            class NewType(objectify.ObjectifiedDataElement):
-                pass
+        def checkMyType(s):
+            return True
 
-            def checkMyType(s):
-                return True
+        pytype = objectify.PyType("mytype", checkMyType, NewType)
+        self.assert_(pytype not in objectify.getRegisteredTypes())
+        pytype.register()
+        self.assert_(pytype in objectify.getRegisteredTypes())
+        pytype.unregister()
+        self.assert_(pytype not in objectify.getRegisteredTypes())
 
-            pytype = objectify.PyType("mytype", checkMyType, NewType)
-            pytype.register()
-            self.assert_(pytype in objectify.getRegisteredTypes())
-            pytype.unregister()
+        pytype.register(before = [objectify.getRegisteredTypes()[0].name])
+        self.assertEquals(pytype, objectify.getRegisteredTypes()[0])
+        pytype.unregister()
 
-            pytype.register(before = [objectify.getRegisteredTypes()[0].name])
-            self.assertEquals(pytype, objectify.getRegisteredTypes()[0])
-            pytype.unregister()
+        pytype.register(after = [objectify.getRegisteredTypes()[0].name])
+        self.assertNotEqual(pytype, objectify.getRegisteredTypes()[0])
+        pytype.unregister()
 
-            pytype.register(after = [objectify.getRegisteredTypes()[0].name])
-            self.assertNotEqual(pytype, objectify.getRegisteredTypes()[0])
-            pytype.unregister()
+        self.assertRaises(ValueError, pytype.register,
+                          before = [objectify.getRegisteredTypes()[0].name],
+                          after  = [objectify.getRegisteredTypes()[1].name])
 
-            self.assertRaises(ValueError, pytype.register,
-                              before = [objectify.getRegisteredTypes()[0].name],
-                              after  = [objectify.getRegisteredTypes()[1].name])
+    def test_registered_type_stringify(self):
+        from datetime import datetime
+        def parse_date(value):
+            if len(value) != 14:
+                raise ValueError(value)
+            Y = int(value[0:4])
+            M = int(value[4:6])
+            D = int(value[6:8])
+            h = int(value[8:10])
+            m = int(value[10:12])
+            s = int(value[12:14])
+            return datetime(Y, M, D, h, m, s)
 
-        finally:
-            for pytype in objectify.getRegisteredTypes():
-                pytype.unregister()
-            for pytype in orig_types:
-                pytype.register()
+        def stringify_date(date):
+            return date.strftime("%Y%m%d%H%M%S")
+
+        class DatetimeElement(objectify.ObjectifiedDataElement):
+            def pyval(self):
+                return parse_date(self.text)
+            pyval = property(pyval)
+
+        datetime_type = objectify.PyType(
+            "datetime", parse_date, DatetimeElement, stringify_date)
+        datetime_type.xmlSchemaTypes = "dateTime"
+        datetime_type.register()
+
+        NAMESPACE = "http://foo.net/xmlns"
+        NAMESPACE_MAP = {'ns': NAMESPACE}
+
+        r = objectify.Element("{%s}root" % NAMESPACE, nsmap=NAMESPACE_MAP)
+        time = datetime.now()
+        r.date = time
+
+        self.assert_(isinstance(r.date, DatetimeElement))
+        self.assert_(isinstance(r.date.pyval, datetime))
+
+        self.assertEquals(r.date.pyval, parse_date(stringify_date(time)))
+        self.assertEquals(r.date.text, stringify_date(time))
+
+        r.date = objectify.E.date(time)
+
+        self.assert_(isinstance(r.date, DatetimeElement))
+        self.assert_(isinstance(r.date.pyval, datetime))
+
+        self.assertEquals(r.date.pyval, parse_date(stringify_date(time)))
+        self.assertEquals(r.date.text, stringify_date(time))
 
     def test_object_path(self):
         root = self.XML(xml_str)
@@ -2182,7 +2260,7 @@ class ObjectifyTestCase(HelperTestCase):
         import pickle
 
         root = self.XML(xml_str)
-        out = StringIO()
+        out = BytesIO()
         pickle.dump(root, out)
 
         new_root = pickle.loads(out.getvalue())
@@ -2194,7 +2272,7 @@ class ObjectifyTestCase(HelperTestCase):
         import pickle
 
         tree = etree.ElementTree(self.XML(xml_str + "<?my pi?>"))
-        out = StringIO()
+        out = BytesIO()
         pickle.dump(tree, out)
 
         new_tree = pickle.loads(out.getvalue())
@@ -2210,11 +2288,6 @@ class ObjectifyTestCase(HelperTestCase):
         root = E.root(E.val(23))
         self.assert_(isinstance(root.val, objectify.IntElement))
 
-    def test_efactory_long(self):
-        E = objectify.E
-        root = E.root(E.val(23L))
-        self.assert_(isinstance(root.val, objectify.LongElement))
-
     def test_efactory_float(self):
         E = objectify.E
         root = E.root(E.val(233.23))
@@ -2227,7 +2300,7 @@ class ObjectifyTestCase(HelperTestCase):
 
     def test_efactory_unicode(self):
         E = objectify.E
-        root = E.root(E.val(unicode("blöödy häll", encoding="ISO-8859-1")))
+        root = E.root(E.val(_str("blöödy häll", encoding="ISO-8859-1")))
         self.assert_(isinstance(root.val, objectify.StringElement))
 
     def test_efactory_bool(self):
@@ -2264,13 +2337,60 @@ class ObjectifyTestCase(HelperTestCase):
         self.assert_(isinstance(root.value[0], objectify.IntElement))
         self.assert_(isinstance(root.value[1], objectify.FloatElement))
 
+    def test_XML_base_url_docinfo(self):
+        root = objectify.XML(_bytes("<root/>"), base_url="http://no/such/url")
+        docinfo = root.getroottree().docinfo
+        self.assertEquals(docinfo.URL, "http://no/such/url")
+ 
+    def test_XML_set_base_url_docinfo(self):
+        root = objectify.XML(_bytes("<root/>"), base_url="http://no/such/url")
+        docinfo = root.getroottree().docinfo
+        self.assertEquals(docinfo.URL, "http://no/such/url")
+        docinfo.URL = "https://secret/url"
+        self.assertEquals(docinfo.URL, "https://secret/url")
+ 
+    def test_parse_stringio_base_url(self):
+        tree = objectify.parse(BytesIO("<root/>"), base_url="http://no/such/url")
+        docinfo = tree.docinfo
+        self.assertEquals(docinfo.URL, "http://no/such/url")
+ 
+    def test_parse_base_url_docinfo(self):
+        tree = objectify.parse(fileInTestDir('include/test_xinclude.xml'),
+                               base_url="http://no/such/url")
+        docinfo = tree.docinfo
+        self.assertEquals(docinfo.URL, "http://no/such/url")
+
+    def test_xml_base(self):
+        root = objectify.XML(_bytes("<root/>"), base_url="http://no/such/url")
+        self.assertEquals(root.base, "http://no/such/url")
+        self.assertEquals(
+            root.get('{http://www.w3.org/XML/1998/namespace}base'), None)
+        root.base = "https://secret/url"
+        self.assertEquals(root.base, "https://secret/url")
+        self.assertEquals(
+            root.get('{http://www.w3.org/XML/1998/namespace}base'),
+            "https://secret/url")
+ 
+    def test_xml_base_attribute(self):
+        root = objectify.XML(_bytes("<root/>"), base_url="http://no/such/url")
+        self.assertEquals(root.base, "http://no/such/url")
+        self.assertEquals(
+            root.get('{http://www.w3.org/XML/1998/namespace}base'), None)
+        root.set('{http://www.w3.org/XML/1998/namespace}base',
+                 "https://secret/url")
+        self.assertEquals(root.base, "https://secret/url")
+        self.assertEquals(
+            root.get('{http://www.w3.org/XML/1998/namespace}base'),
+            "https://secret/url")
+
+
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTests([unittest.makeSuite(ObjectifyTestCase)])
     if sys.version_info >= (2,4):
         suite.addTests(
-            [doctest.DocFileSuite('../../../doc/objectify.txt')])
+            [make_doctest('../../../doc/objectify.txt')])
     return suite
 
 if __name__ == '__main__':
-    print 'to test use test.py %s' % __file__
+    print('to test use test.py %s' % __file__)
