@@ -4,10 +4,14 @@
 Test cases related to XPath evaluation and the XPath class
 """
 
-import unittest, doctest
-from StringIO import StringIO
+import unittest, sys, os.path
 
-from common_imports import etree, HelperTestCase
+this_dir = os.path.dirname(__file__)
+if this_dir not in sys.path:
+    sys.path.insert(0, this_dir) # needed for Py3
+
+from common_imports import etree, HelperTestCase, _bytes, BytesIO
+from common_imports import doctest, make_doctest
 
 class ETreeXPathTestCase(HelperTestCase):
     """XPath tests etree"""
@@ -23,7 +27,7 @@ class ETreeXPathTestCase(HelperTestCase):
                           tree.xpath('number(/a)'))
         tree = self.parse('<a>A</a>')
         actual = str(tree.xpath('number(/a)'))
-        expected = ['nan', '1.#qnan']
+        expected = ['nan', '1.#qnan', 'nanq']
         if not actual.lower() in expected:
             self.fail('Expected a NAN value, got %s' % actual)
         
@@ -67,15 +71,68 @@ class ETreeXPathTestCase(HelperTestCase):
         self.assertEquals(['Foo', 'Bar'],
                           tree.xpath('/a/b/text()'))
 
+    def test_xpath_list_text_parent(self):
+        tree = self.parse('<a><b>FooBar</b><b>BarFoo</b></a>')
+        root = tree.getroot()
+        self.assertEquals(['FooBar', 'BarFoo'],
+                          tree.xpath('/a/b/text()'))
+        self.assertEquals([root[0], root[1]],
+                          [r.getparent() for r in tree.xpath('/a/b/text()')])
+
+    def test_xpath_list_text_parent_no_smart_strings(self):
+        tree = self.parse('<a><b>FooBar</b><b>BarFoo</b></a>')
+        root = tree.getroot()
+        self.assertEquals(['FooBar', 'BarFoo'],
+                          tree.xpath('/a/b/text()', smart_strings=True))
+        self.assertEquals([root[0], root[1]],
+                          [r.getparent() for r in
+                           tree.xpath('/a/b/text()', smart_strings=True)])
+
+        self.assertEquals(['FooBar', 'BarFoo'],
+                          tree.xpath('/a/b/text()', smart_strings=False))
+        self.assertEquals([False, False],
+                          [hasattr(r, 'getparent') for r in
+                           tree.xpath('/a/b/text()', smart_strings=False)])
+
+    def test_xpath_list_unicode_text_parent(self):
+        xml = _bytes('<a><b>FooBar\\u0680\\u3120</b><b>BarFoo\\u0680\\u3120</b></a>').decode("unicode_escape")
+        tree = self.parse(xml.encode('utf-8'))
+        root = tree.getroot()
+        self.assertEquals([_bytes('FooBar\\u0680\\u3120').decode("unicode_escape"),
+                           _bytes('BarFoo\\u0680\\u3120').decode("unicode_escape")],
+                          tree.xpath('/a/b/text()'))
+        self.assertEquals([root[0], root[1]],
+                          [r.getparent() for r in tree.xpath('/a/b/text()')])
+
     def test_xpath_list_attribute(self):
         tree = self.parse('<a b="B" c="C"/>')
         self.assertEquals(['B'],
                           tree.xpath('/a/@b'))
 
+    def test_xpath_list_attribute_parent(self):
+        tree = self.parse('<a b="BaSdFgHjKl" c="CqWeRtZuI"/>')
+        results = tree.xpath('/a/@c')
+        self.assertEquals(1, len(results))
+        self.assertEquals('CqWeRtZuI', results[0])
+        self.assertEquals(tree.getroot().tag, results[0].getparent().tag)
+
+    def test_xpath_list_attribute_parent_no_smart_strings(self):
+        tree = self.parse('<a b="BaSdFgHjKl" c="CqWeRtZuI"/>')
+
+        results = tree.xpath('/a/@c', smart_strings=True)
+        self.assertEquals(1, len(results))
+        self.assertEquals('CqWeRtZuI', results[0])
+        self.assertEquals(tree.getroot().tag, results[0].getparent().tag)
+
+        results = tree.xpath('/a/@c', smart_strings=False)
+        self.assertEquals(1, len(results))
+        self.assertEquals('CqWeRtZuI', results[0])
+        self.assertEquals(False, hasattr(results[0], 'getparent'))
+
     def test_xpath_list_comment(self):
         tree = self.parse('<a><!-- Foo --></a>')
         self.assertEquals(['<!-- Foo -->'],
-                          map(repr, tree.xpath('/a/node()')))
+                          list(map(repr, tree.xpath('/a/node()'))))
 
     def test_rel_xpath_boolean(self):
         root = etree.XML('<a><b><c/></b></a>')
@@ -97,24 +154,44 @@ class ETreeXPathTestCase(HelperTestCase):
         root = tree.getroot()
         self.assertEquals(
             [root[0]],
-            tree.xpath('//foo:b', {'foo': 'uri:a'}))
+            tree.xpath('//foo:b', namespaces={'foo': 'uri:a'}))
         self.assertEquals(
             [],
-            tree.xpath('//foo:b', {'foo': 'uri:c'}))
+            tree.xpath('//foo:b', namespaces={'foo': 'uri:c'}))
         self.assertEquals(
             [root[0]],
-            root.xpath('//baz:b', {'baz': 'uri:a'}))
+            root.xpath('//baz:b', namespaces={'baz': 'uri:a'}))
 
     def test_xpath_ns_none(self):
         tree = self.parse('<a xmlns="uri:a"><b></b></a>')
         root = tree.getroot()
         self.assertRaises(
             TypeError,
-            root.xpath, '//b', {None: 'uri:a'})
+            root.xpath, '//b', namespaces={None: 'uri:a'})
+
+    def test_xpath_ns_empty(self):
+        tree = self.parse('<a xmlns="uri:a"><b></b></a>')
+        root = tree.getroot()
+        self.assertRaises(
+            TypeError,
+            root.xpath, '//b', namespaces={'': 'uri:a'})
 
     def test_xpath_error(self):
         tree = self.parse('<a/>')
-        self.assertRaises(SyntaxError, tree.xpath, '\\fad')
+        self.assertRaises(etree.XPathEvalError, tree.xpath, '\\fad')
+
+    def test_xpath_class_error(self):
+        self.assertRaises(SyntaxError, etree.XPath, '\\fad')
+        self.assertRaises(etree.XPathSyntaxError, etree.XPath, '\\fad')
+
+    def test_xpath_prefix_error(self):
+        tree = self.parse('<a/>')
+        self.assertRaises(etree.XPathEvalError, tree.xpath, '/fa:d')
+
+    def test_xpath_class_prefix_error(self):
+        tree = self.parse('<a/>')
+        xpath = etree.XPath("/fa:d")
+        self.assertRaises(etree.XPathEvalError, xpath, tree)
 
     def test_elementtree_getpath(self):
         a  = etree.Element("a")
@@ -148,7 +225,7 @@ class ETreeXPathTestCase(HelperTestCase):
         root = tree.getroot()
         self.assertEquals(
             [root],
-            e.evaluate('//a'))
+            e('//a'))
 
     def test_xpath_evaluator_tree(self):
         tree = self.parse('<a><b><c></c></b></a>')
@@ -156,11 +233,26 @@ class ETreeXPathTestCase(HelperTestCase):
         e = etree.XPathEvaluator(child_tree)
         self.assertEquals(
             [],
-            e.evaluate('a'))
+            e('a'))
         root = child_tree.getroot()
         self.assertEquals(
             [root[0]],
-            e.evaluate('c'))
+            e('c'))
+
+    def test_xpath_evaluator_tree_absolute(self):
+        tree = self.parse('<a><b><c></c></b></a>')
+        child_tree = etree.ElementTree(tree.getroot()[0])
+        e = etree.XPathEvaluator(child_tree)
+        self.assertEquals(
+            [],
+            e('/a'))
+        root = child_tree.getroot()
+        self.assertEquals(
+            [root],
+            e('/b'))
+        self.assertEquals(
+            [],
+            e('/c'))
 
     def test_xpath_evaluator_element(self):
         tree = self.parse('<a><b><c></c></b></a>')
@@ -168,16 +260,16 @@ class ETreeXPathTestCase(HelperTestCase):
         e = etree.XPathEvaluator(root[0])
         self.assertEquals(
             [root[0][0]],
-            e.evaluate('c'))
+            e('c'))
         
     def test_xpath_extensions(self):
         def foo(evaluator, a):
             return 'hello %s' % a
         extension = {(None, 'foo'): foo}
         tree = self.parse('<a><b></b></a>')
-        e = etree.XPathEvaluator(tree, None, [extension])
+        e = etree.XPathEvaluator(tree, extensions=[extension])
         self.assertEquals(
-            "hello you", e.evaluate("foo('you')"))
+            "hello you", e("foo('you')"))
 
     def test_xpath_extensions_wrong_args(self):
         def foo(evaluator, a, b):
@@ -185,15 +277,15 @@ class ETreeXPathTestCase(HelperTestCase):
         extension = {(None, 'foo'): foo}
         tree = self.parse('<a><b></b></a>')
         e = etree.XPathEvaluator(tree, extensions=[extension])
-        self.assertRaises(TypeError, e.evaluate, "foo('you')")
+        self.assertRaises(TypeError, e, "foo('you')")
 
     def test_xpath_extensions_error(self):
         def foo(evaluator, a):
             return 1/0
         extension = {(None, 'foo'): foo}
         tree = self.parse('<a/>')
-        e = etree.XPathEvaluator(tree, None, [extension])
-        self.assertRaises(ZeroDivisionError, e.evaluate, "foo('test')")
+        e = etree.XPathEvaluator(tree, extensions=[extension])
+        self.assertRaises(ZeroDivisionError, e, "foo('test')")
 
     def test_xpath_extensions_nodes(self):
         def f(evaluator, arg):
@@ -205,8 +297,8 @@ class ETreeXPathTestCase(HelperTestCase):
             return r
 
         x = self.parse('<a/>')
-        e = etree.XPathEvaluator(x, None, [{(None, 'foo'): f}])
-        r = e.evaluate("foo('World')/result")
+        e = etree.XPathEvaluator(x, extensions=[{(None, 'foo'): f}])
+        r = e("foo('World')/result")
         self.assertEquals(2, len(r))
         self.assertEquals('Hoi', r[0].text)
         self.assertEquals('Dag', r[1].text)
@@ -221,8 +313,8 @@ class ETreeXPathTestCase(HelperTestCase):
             return r
 
         x = self.parse('<a/>')
-        e = etree.XPathEvaluator(x, None, [{(None, 'foo'): f}])
-        r = e.evaluate("foo(/*)/result")
+        e = etree.XPathEvaluator(x, extensions=[{(None, 'foo'): f}])
+        r = e("foo(/*)/result")
         self.assertEquals(2, len(r))
         self.assertEquals('Hoi', r[0].text)
         self.assertEquals('Dag', r[1].text)
@@ -238,26 +330,92 @@ class ETreeXPathTestCase(HelperTestCase):
             return r
 
         x = self.parse('<result>Honk</result>')
-        e = etree.XPathEvaluator(x, None, [{(None, 'foo'): f}])
-        r = e.evaluate("foo(/*)/result")
+        e = etree.XPathEvaluator(x, extensions=[{(None, 'foo'): f}])
+        r = e("foo(/*)/result")
         self.assertEquals(3, len(r))
         self.assertEquals('Hoi',  r[0].text)
         self.assertEquals('Dag',  r[1].text)
         self.assertEquals('Honk', r[2].text)
 
+    def test_xpath_context_node(self):
+        tree = self.parse('<root><a/><b><c/></b></root>')
+
+        check_call = []
+        def check_context(ctxt, nodes):
+            self.assertEquals(len(nodes), 1)
+            check_call.append(nodes[0].tag)
+            self.assertEquals(ctxt.context_node, nodes[0])
+            return True
+
+        find = etree.XPath("//*[p:foo(.)]",
+                           namespaces={'p' : 'ns'},
+                           extensions=[{('ns', 'foo') : check_context}])
+        find(tree)
+
+        check_call.sort()
+        self.assertEquals(check_call, ["a", "b", "c", "root"])
+
+    def test_xpath_eval_context_propagation(self):
+        tree = self.parse('<root><a/><b><c/></b></root>')
+
+        check_call = {}
+        def check_context(ctxt, nodes):
+            self.assertEquals(len(nodes), 1)
+            tag = nodes[0].tag
+            # empty during the "b" call, a "b" during the "c" call
+            check_call[tag] = ctxt.eval_context.get("b")
+            ctxt.eval_context[tag] = tag
+            return True
+
+        find = etree.XPath("//b[p:foo(.)]/c[p:foo(.)]",
+                           namespaces={'p' : 'ns'},
+                           extensions=[{('ns', 'foo') : check_context}])
+        result = find(tree)
+
+        self.assertEquals(result, [tree.getroot()[1][0]])
+        self.assertEquals(check_call, {'b':None, 'c':'b'})
+
+    def test_xpath_eval_context_clear(self):
+        tree = self.parse('<root><a/><b><c/></b></root>')
+
+        check_call = {}
+        def check_context(ctxt):
+            check_call["done"] = True
+            # context must be empty for each new evaluation
+            self.assertEquals(len(ctxt.eval_context), 0)
+            ctxt.eval_context["test"] = True
+            return True
+
+        find = etree.XPath("//b[p:foo()]",
+                           namespaces={'p' : 'ns'},
+                           extensions=[{('ns', 'foo') : check_context}])
+        result = find(tree)
+
+        self.assertEquals(result, [tree.getroot()[1]])
+        self.assertEquals(check_call["done"], True)
+
+        check_call.clear()
+        find = etree.XPath("//b[p:foo()]",
+                           namespaces={'p' : 'ns'},
+                           extensions=[{('ns', 'foo') : check_context}])
+        result = find(tree)
+
+        self.assertEquals(result, [tree.getroot()[1]])
+        self.assertEquals(check_call["done"], True)
+
     def test_xpath_variables(self):
         x = self.parse('<a attr="true"/>')
         e = etree.XPathEvaluator(x)
 
         expr = "/a[@attr=$aval]"
-        r = e.evaluate(expr, aval=1)
+        r = e(expr, aval=1)
         self.assertEquals(0, len(r))
 
-        r = e.evaluate(expr, aval="true")
+        r = e(expr, aval="true")
         self.assertEquals(1, len(r))
         self.assertEquals("true", r[0].get('attr'))
 
-        r = e.evaluate(expr, aval=True)
+        r = e(expr, aval=True)
         self.assertEquals(1, len(r))
         self.assertEquals("true", r[0].get('attr'))
 
@@ -268,7 +426,7 @@ class ETreeXPathTestCase(HelperTestCase):
         element = etree.Element("test-el")
         etree.SubElement(element, "test-sub")
         expr = "$value"
-        r = e.evaluate(expr, value=element)
+        r = e(expr, value=element)
         self.assertEquals(1, len(r))
         self.assertEquals(element.tag, r[0].tag)
         self.assertEquals(element[0].tag, r[0][0].tag)
@@ -299,41 +457,41 @@ class ETreeXPathTestCase(HelperTestCase):
         e = etree.XPathEvaluator(x, extensions=[extension])
         del x
 
-        self.assertRaises(LocalException, e.evaluate, "foo(., 0)")
-        self.assertRaises(LocalException, e.evaluate, "foo(., $value)", value=0)
+        self.assertRaises(LocalException, e, "foo(., 0)")
+        self.assertRaises(LocalException, e, "foo(., $value)", value=0)
 
-        r = e.evaluate("foo(., $value)", value=1)
+        r = e("foo(., $value)", value=1)
         self.assertEqual(len(r), 0)
 
-        r = e.evaluate("foo(.,  1)")
+        r = e("foo(.,  1)")
         self.assertEqual(len(r), 0)
 
-        r = e.evaluate("foo(., $value)", value=2)
+        r = e("foo(., $value)", value=2)
         self.assertEqual(len(r), 0)
 
-        r = e.evaluate("foo(., $value)", value=3)
+        r = e("foo(., $value)", value=3)
         self.assertEqual(len(r), 1)
         self.assertEqual(r[0].tag, "test")
 
-        r = e.evaluate("foo(., $value)", value="false")
+        r = e("foo(., $value)", value="false")
         self.assertEqual(len(r), 1)
         self.assertEqual(r[0].tag, "NODE")
 
-        r = e.evaluate("foo(., 'false')")
+        r = e("foo(., 'false')")
         self.assertEqual(len(r), 1)
         self.assertEqual(r[0].tag, "NODE")
 
-        r = e.evaluate("foo(., 'true')")
+        r = e("foo(., 'true')")
         self.assertEqual(len(r), 1)
         self.assertEqual(r[0].tag, "a")
         self.assertEqual(r[0][0].tag, "test")
 
-        r = e.evaluate("foo(., $value)", value="true")
+        r = e("foo(., $value)", value="true")
         self.assertEqual(len(r), 1)
         self.assertEqual(r[0].tag, "a")
 
-        self.assertRaises(LocalException, e.evaluate, "foo(., 0)")
-        self.assertRaises(LocalException, e.evaluate, "foo(., $value)", value=0)
+        self.assertRaises(LocalException, e, "foo(., 0)")
+        self.assertRaises(LocalException, e, "foo(., $value)", value=0)
 
 
 class ETreeXPathClassTestCase(HelperTestCase):
@@ -342,15 +500,15 @@ class ETreeXPathClassTestCase(HelperTestCase):
         x = self.parse('<a attr="true"/>')
 
         expr = etree.XPath("/a[@attr != 'true']")
-        r = expr.evaluate(x)
+        r = expr(x)
         self.assertEquals(0, len(r))
 
         expr = etree.XPath("/a[@attr = 'true']")
-        r = expr.evaluate(x)
+        r = expr(x)
         self.assertEquals(1, len(r))
 
         expr = etree.XPath( expr.path )
-        r = expr.evaluate(x)
+        r = expr(x)
         self.assertEquals(1, len(r))
 
     def test_xpath_compile_element(self):
@@ -358,22 +516,22 @@ class ETreeXPathClassTestCase(HelperTestCase):
         root = x.getroot()
 
         expr = etree.XPath("./b")
-        r = expr.evaluate(root)
+        r = expr(root)
         self.assertEquals(1, len(r))
         self.assertEquals('b', r[0].tag)
 
         expr = etree.XPath("./*")
-        r = expr.evaluate(root)
+        r = expr(root)
         self.assertEquals(2, len(r))
 
     def test_xpath_compile_vars(self):
         x = self.parse('<a attr="true"/>')
 
         expr = etree.XPath("/a[@attr=$aval]")
-        r = expr.evaluate(x, aval=False)
+        r = expr(x, aval=False)
         self.assertEquals(0, len(r))
 
-        r = expr.evaluate(x, aval=True)
+        r = expr(x, aval=True)
         self.assertEquals(1, len(r))
 
     def test_xpath_compile_error(self):
@@ -388,29 +546,30 @@ class ETreeETXPathClassTestCase(HelperTestCase):
         x = self.parse('<a><b xmlns="nsa"/><b xmlns="nsb"/></a>')
 
         expr = etree.ETXPath("/a/{nsa}b")
-        r = expr.evaluate(x)
+        r = expr(x)
         self.assertEquals(1, len(r))
         self.assertEquals('{nsa}b', r[0].tag)
 
         expr = etree.ETXPath("/a/{nsb}b")
-        r = expr.evaluate(x)
+        r = expr(x)
         self.assertEquals(1, len(r))
         self.assertEquals('{nsb}b', r[0].tag)
 
     def test_xpath_compile_unicode(self):
-        x = self.parse(u'<a><b xmlns="nsa\uf8d2"/><b xmlns="nsb\uf8d1"/></a>')
+        x = self.parse(_bytes('<a><b xmlns="nsa\\uf8d2"/><b xmlns="nsb\\uf8d1"/></a>'
+                              ).decode("unicode_escape"))
 
-        expr = etree.ETXPath(u"/a/{nsa\uf8d2}b")
-        r = expr.evaluate(x)
+        expr = etree.ETXPath(_bytes("/a/{nsa\\uf8d2}b").decode("unicode_escape"))
+        r = expr(x)
         self.assertEquals(1, len(r))
-        self.assertEquals(u'{nsa\uf8d2}b', r[0].tag)
+        self.assertEquals(_bytes('{nsa\\uf8d2}b').decode("unicode_escape"), r[0].tag)
 
-        expr = etree.ETXPath(u"/a/{nsb\uf8d1}b")
-        r = expr.evaluate(x)
+        expr = etree.ETXPath(_bytes("/a/{nsb\\uf8d1}b").decode("unicode_escape"))
+        r = expr(x)
         self.assertEquals(1, len(r))
-        self.assertEquals(u'{nsb\uf8d1}b', r[0].tag)
+        self.assertEquals(_bytes('{nsb\\uf8d1}b').decode("unicode_escape"), r[0].tag)
 
-SAMPLE_XML = etree.parse(StringIO("""
+SAMPLE_XML = etree.parse(BytesIO("""
 <body>
   <tag>text</tag>
   <section>
@@ -440,7 +599,7 @@ def setTest2(ctxt, st1):
     return st1[0:2]
 
 def argsTest1(ctxt, s, f, b, st):
-    return ", ".join(map(str, (s, f, b, map(tag, st))))
+    return ", ".join(map(str, (s, f, b, list(map(tag, st)))))
 
 def argsTest2(ctxt, st1, st2):
     st1.extend(st2)
@@ -469,37 +628,44 @@ def xpath():
     Test xpath extension functions.
     
     >>> root = SAMPLE_XML
-    >>> e = etree.XPathEvaluator(root, None, [extension])
-    >>> e.evaluate("stringTest('you')")
+    >>> e = etree.XPathEvaluator(root, extensions=[extension])
+    >>> e("stringTest('you')")
     'Hello you'
-    >>> e.evaluate(u"stringTest('\xe9lan')")
+    >>> e(_bytes("stringTest('\\\\xe9lan')").decode("unicode_escape"))
     u'Hello \\xe9lan'
-    >>> e.evaluate("stringTest('you','there')")
+    >>> e("stringTest('you','there')")
     Traceback (most recent call last):
     ...
     TypeError: stringTest() takes exactly 2 arguments (3 given)
-    >>> e.evaluate("floatTest(2)")
+    >>> e("floatTest(2)")
     6.0
-    >>> e.evaluate("booleanTest(true())")
+    >>> e("booleanTest(true())")
     False
-    >>> map(tag, e.evaluate("setTest(/body/tag)"))
+    >>> list(map(tag, e("setTest(/body/tag)")))
     ['tag']
-    >>> map(tag, e.evaluate("setTest2(/body/*)"))
+    >>> list(map(tag, e("setTest2(/body/*)")))
     ['tag', 'section']
-    >>> e.evaluate("argsTest1('a',1.5,true(),/body/tag)")
+    >>> e("argsTest1('a',1.5,true(),/body/tag)")
     "a, 1.5, True, ['tag', 'tag', 'tag']"
-    >>> map(tag, e.evaluate("argsTest2(/body/tag, /body/section)"))
+    >>> list(map(tag, e("argsTest2(/body/tag, /body/section)")))
     ['tag', 'section', 'tag', 'tag']
-    >>> e.evaluate("resultTypesTest()")
+    >>> e("resultTypesTest()")
     Traceback (most recent call last):
     ...
-    XPathResultError: This is not a node: x
+    XPathResultError: This is not a node: 'x'
     >>> try:
-    ...     e.evaluate("resultTypesTest2()")
+    ...     e("resultTypesTest2()")
     ... except etree.XPathResultError:
-    ...     print "Got error"
+    ...     print("Got error")
     Got error
     """
+
+if sys.version_info[0] >= 3:
+    xpath.__doc__ = xpath.__doc__.replace(" u'", " '")
+    xpath.__doc__ = xpath.__doc__.replace(" XPathResultError",
+                                          " lxml.etree.XPathResultError")
+    xpath.__doc__ = xpath.__doc__.replace(" exactly 2 arguments",
+                                          " exactly 2 positional arguments")
    
 def test_suite():
     suite = unittest.TestSuite()
@@ -508,8 +674,8 @@ def test_suite():
     suite.addTests([unittest.makeSuite(ETreeETXPathClassTestCase)])
     suite.addTests([doctest.DocTestSuite()])
     suite.addTests(
-        [doctest.DocFileSuite('../../../doc/xpathxslt.txt')])
+        [make_doctest('../../../doc/xpathxslt.txt')])
     return suite
 
 if __name__ == '__main__':
-    unittest.main()
+    print('to test use test.py %s' % __file__)
